/* vxwr.c
**
** vxwr: Append data from stdin to a VxMail-like file (data file + index file);
** vxrd: Extract data from a VxMail-like file (data file + index file);
** vxrm: Remove messages from a VxMail-like file (data file + index file).
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>

#include "prefs.h"
#include "vxerror.h"
#include "tools.h"
#include "version.h"
#include "folder.h"
#include "vxopt.h"
#include "mrmacs.h"
#include "inline-funcs.h"

static void usage (void)
{
    if (!strcmp (progname, "vxrd")) {
	fprintf (stderr, "\nUsage: %s file message-# ..."
			 "\n       %s -h"
			 "\n       %s -V\n"
			 "\n   -h  - print this text"
			 "\n   -V  - print the version number\n\n",
			 progname, progname, progname);
    } else if (!strcmp (progname, "vxwr")) {
	fprintf (stderr, "\nUsage: %s [-x|-u] file"
			 "\n       %s -h"
			 "\n       %s -V\n"
			 "\n   -h  - print this text"
			 "\n   -V  - print the version number"
			 "\n   -u  - truncate the file"
			 "\n   -x  - assume that the index file was newly "
			 "created has one\n\n",
			 progname, progname, progname);
    } else if (!strcmp (progname, "vxrm")) {
	fprintf (stderr, "\nUsage: %s [-x|-u] file message-# ..."
			 "\n       %s -h"
			 "\n       %s -V\n"
			 "\n   -h  - print this text"
			 "\n   -V  - print the version number"
			 "\n   -u  - truncate the file"
			 "\n   -x  - assume that the index file was newly "
			 "created has one\n\n",
			 progname, progname, progname);
    } else {
	fprintf (stderr, "\n%s: WRONG NAME!\n"
			 "\nUsage: vxrd file message-# ..."
			 "\n       vxwr [-x|-u] file"
			 "\n       vxrm [-x|-u] file message-# ...\n\n",
			 progname);
    }
    exit (EX_USAGE);
}

static int vxwr (int argc, char *argv[]);
static int vxrd (int argc, char *argv[]);
static int vxrm (int argc, char *argv[]);

int main (int argc, char *argv[])
{
    progname = strrchr (argv[0], '/');
    ifnull (progname) progname = argv[0]; else ++progname;
    if (!strcmp (progname, "vxrd")) { return vxrd (argc, argv); }
    if (!strcmp (progname, "vxwr")) { return vxwr (argc, argv); }
    if (!strcmp (progname, "vxrm")) { return vxrm (argc, argv); }
    usage ();
    return 0;
}

static int vxrm (int argc, char *argv[])
{
    vxerror (0, 0, "not implemented (yet)!\n");
    return EX_SOFTWARE;
}

static int vxwr (int argc, char *argv[])
{
    char line[1024];
    int c, mkindex = 0, umode = 0;
    xFile af, bf = (xFile) 0;

    for (;;) {
	c = getopt (argc, argv, "hVx");
	if (c == -1) break;
	switch (c) {
	    case 'h':
		usage();
	    case 'V':
		fprintf (stderr, "%s\n", VXMAIL_VERSION);
		exit (0);
	    case 'u':
		if (mkindex) {
		    vxerror (EX_USAGE, 0,
			     "Please don't supply -u and -x together!\n");
		}
		umode = 1; break;
	    case 'x':
		if (mkindex) {
		    vxerror (EX_USAGE, 0,
			     "Please don't supply -u and -x together!\n");
		}
		mkindex = 1; break;
	    default:
		vxerror (EX_USAGE, 0,
			 "unknown option `%s'\nTry `%s -h' for help, please!\n",
			 argv[optind], progname);
	}
    }
    if (optind >= argc) {
	vxerror (EX_USAGE, 0,
		 "missing file argument - try `%s -h' for help, please!\n",
		 progname);
    }
    if (mkindex) {
	unlessnull (bf = flOpen (argv[optind], "ix", 0600)) {
	    xSet (bf, xSize (bf));
	}
    }
    ifnull (af = flOpen (argv[optind], (umode ? "ux" : "wx"), 0600)) {
	int exc = (errno == EPERM || errno == EACCES ? EX_NOPERM : EX_OSERR);
	vxerror_at (exc, errno, __FILE__, __LINE__ - 2,
		    "writing `%s' failed", argv[optind]);
    }
    unlessnull (bf) { xClose (bf); }
    while (noNULL (fgets (line, 1024, stdin))) {
	xPuts (line, af);
    }
    xSet (af);
    xClose (af);
    return 0;
}

static int vxrd (int argc, char *argv[])
{
    int c;
    long msgno;
    char line[1024], *p;
    xFile af;
    for (;;) {
	c = getopt (argc, argv, "hVx");
	if (c == -1) break;
	switch (c) {
	    case 'h':
		usage();
	    case 'V':
		fprintf (stderr, "%s\n", VXMAIL_VERSION);
		exit (0);
	    default:
		vxerror (EX_USAGE, 0,
			 "unknown option `%s'\nTry `%s -h' for help, please!\n",
			 argv[optind], progname);
	}
    }
    if (argc - optind < 2) {
	vxerror (EX_USAGE, 0,
		 "missing file and/or msg-# argument(s) - "
		 "try `%s -h' for help, please!\n", progname);
    }
    ifnull (af = flOpen (argv[optind], "rs")) {
	vxerror_at (EX_OSERR, errno, __FILE__, __LINE__,
		    "reading `%s' failed", argv[optind]);
    }
    for (++optind; optind < argc; ++optind) {
	msgno = strtol (argv[optind], &p, 0);
	if (*p) {
	    vxerror (0, 0, "ignoring `%s' - which is no number!\n",
			     argv[optind]);
	    continue;
	}
	if (xSeek (af, (off_t) msgno) < 0) {
	    vxerror (0, 0, "skipping message-# `%d' - which is out of range!\n",
		     msgno);
	    continue;
	}
	while (xGets (line, 1024, af) > 0) { fputs (line, stdout); }
    }
    xClose (af);
    return 0;
}
